P0 fixes (blocking usability): - FIX-01: thread-local cuBLAS handle (was creating/destroying per matmul) - FIX-16: EOS token no longer leaks into API responses - FIX-17: max_seq_len configurable via --max-seq-len (default 2048, was hardcoded 256) - FIX-18: max_tokens clamped to available seq space, prompt overflow returns 400 P1 fixes (bugs & performance): - FIX-07: CachingAllocator wired into all hot paths (to_device, embedding, rope, concat) - FIX-08: CudaDeviceProp buffer increased to 32KB for CUDA 12.9 safety - FIX-09: tokenizer byte_fallback graceful degradation (was panic) - FIX-19: causal mask uses -INFINITY instead of -1e9 (BF16 supports inf) - FIX-20: LayerNorm rewritten to numerically stable two-pass algorithm - FIX-21: min block size guard (32 threads) for LayerNorm/RMSNorm launches P2 fixes (improvements): - FIX-22: Option<GpuKVCache> + take() eliminates dummy KV cache allocations - FIX-23: RoPE cache no longer artificially capped at 8192 positions Verified on dash5 (RTX 5090): 51 tok/s batch=1, 74 tok/s 2-concurrent, 1.7-3.3x HF transformers. Co-Authored-By: Claude Opus 4.6 (1M context) <noreply@anthropic.com>
53 lines
1.6 KiB
Plaintext
53 lines
1.6 KiB
Plaintext
#include <cuda_bf16.h>
|
||
|
||
// Apply causal mask: set scores[row][col] = -inf where col > row + offset.
|
||
// offset is used for KV cache: when query starts at position `offset`,
|
||
// we allow attending to positions [0, offset + row].
|
||
// scores: [batch, rows, cols] (flattened batch×heads)
|
||
|
||
__global__ void causal_mask_f32(
|
||
float* __restrict__ scores,
|
||
int rows, int cols, int offset
|
||
) {
|
||
int batch_idx = blockIdx.z;
|
||
int row = blockIdx.y;
|
||
int col = blockIdx.x * blockDim.x + threadIdx.x;
|
||
|
||
if (col < cols && col > row + offset) {
|
||
scores[batch_idx * rows * cols + row * cols + col] = -INFINITY;
|
||
}
|
||
}
|
||
|
||
__global__ void causal_mask_bf16(
|
||
__nv_bfloat16* __restrict__ scores,
|
||
int rows, int cols, int offset
|
||
) {
|
||
int batch_idx = blockIdx.z;
|
||
int row = blockIdx.y;
|
||
int col = blockIdx.x * blockDim.x + threadIdx.x;
|
||
|
||
if (col < cols && col > row + offset) {
|
||
scores[batch_idx * rows * cols + row * cols + col] = __float2bfloat16(-INFINITY);
|
||
}
|
||
}
|
||
|
||
extern "C" {
|
||
|
||
void launch_causal_mask_f32(void* scores, int batch, int rows, int cols,
|
||
int offset, void* stream) {
|
||
int block = 256;
|
||
dim3 grid((cols + block - 1) / block, rows, batch);
|
||
causal_mask_f32<<<grid, block, 0, (cudaStream_t)stream>>>(
|
||
(float*)scores, rows, cols, offset);
|
||
}
|
||
|
||
void launch_causal_mask_bf16(void* scores, int batch, int rows, int cols,
|
||
int offset, void* stream) {
|
||
int block = 256;
|
||
dim3 grid((cols + block - 1) / block, rows, batch);
|
||
causal_mask_bf16<<<grid, block, 0, (cudaStream_t)stream>>>(
|
||
(__nv_bfloat16*)scores, rows, cols, offset);
|
||
}
|
||
|
||
}
|